#include <string.h>
#include "hm.h"

static size_t hash(char *key, size_t table_capacity) {
    size_t h = 0;
    for (size_t i = 0; i < strlen(key); ++i) {
        h = (h + key[i] * 43) % table_capacity;
    }
    return h;
}

void hm_init(hm_t *hm, size_t table_capacity) {
    hm->table = malloc(sizeof(*(hm->table)) * table_capacity);
    hm->table_capacity = table_capacity;
    hm->table_length = 0;
    for (size_t i = 0; i < hm->table_capacity; ++i) {
        hm->table[i].state = EMPTY;
    }
}

void hm_destroy(hm_t *hm) {
    free(hm->table);
    hm->table = NULL;
    hm->table_capacity = 0;
    hm->table_length = 0;
}

void hm_insert(hm_t *hm, char *key, char *value) {
    if (hm->table_capacity == hm->table_length) {
        return;
    }
    size_t index = hash(key, hm->table_capacity);
    while(hm->table[index].state == OCCUPIED && (0 != strcmp(key, hm->table[index].key))) {
        index = (index + 1) % hm->table_capacity;
    }
    if (hm->table[index].state != OCCUPIED) {
        hm->table_length += 1;
        strcpy(hm->table[index].key, key);
        hm->table[index].state = OCCUPIED;
    }
    strcpy(hm->table[index].value, value);
}

void hm_remove(hm_t *hm, char *key) {
    if (0 == hm->table_length) {
        return;
    }
    size_t index = hash(key, hm->table_capacity);
    while (hm->table[index].state == EMPTY && 
        (0 != strcmp(key, hm->table[index].key))) {
        index = (index + 1) % hm->table_capacity;
    }
    if (0 == strcmp(key, hm->table[index].key)) {
        hm->table[index].state = DELETED;
        hm->table[index].key[0] = '\0';
        hm->table_length -= 1;
    }
}

